# Analysing egg shape variation 

# Script written by Emma Sherratt (emma.sherratt@gmail.com)
# Written May 2016, updated May 2018
# Accompanying Attard et al. 2018 PeerJ

library(geomorph) # using version 3.0.6 
library(Morpho) # needed for rotating eggs by principal axes and reading in binary ply files
library(geometry) # needed for convex hull volume calculation

# Working directory should contain the "NTS" folder of .nts files. Set to sourcefile location.

## Procrustes Superimposition
## Read in coordinate (.nts) files
ntsfilelist <-  list.files(path= "NTS", pattern= ".nts")
speclist <- gsub(".nts", "", ntsfilelist) # get specimen names from file names
ntsfilelist <- paste("NTS/", ntsfilelist, sep="")
eggdata <- readmulti.nts(ntsfilelist) # read in .nts files
egg.gpa <- gpagen(eggdata) # 1st Procrustes Superimposition simply to grossly align eggs

# Procrustes Superimposition, using Bending Energy and setting all semis to be sliding [surfaces = c(1:206)]
egg.gpa.surfs <- gpagen(egg.gpa$coords, surfaces = c(1:206), max.iter = 15, ProcD = FALSE) # takes a while
# Rename specimens, removing path info
  dimnames(egg.gpa.surfs$coords)[[3]] <- speclist

# Make a classifiers matrix
  # separates the specimen names by underscore & coerce into a matrix
  classifiers <- matrix(unlist(strsplit(dimnames(egg.gpa.surfs$coords)[[3]], "_")), ncol=4, byrow=T)
  rownames(classifiers) <- dimnames(egg.gpa.surfs$coords)[[3]]
  # reorder classifiers table to match order we wish to have species presented in
  # "Colluricincia","Neochmia","Acanthagenys","Malurus"
  # Grey shrike-thrush, Red-browed finch, Spiny-cheeked honeyeater, Superb fairy-wren 
  classifiers <- rbind(classifiers[which(classifiers[,1] == "Colluricincia"),],
                       classifiers[which(classifiers[,1] == "Neochmia"),],
                       classifiers[which(classifiers[,1] == "Acanthagenys"),],
                       classifiers[which(classifiers[,1] == "Malurus"),])
  # Isolate classifiers for ease of use later on 
  species <- factor(classifiers[,1], levels=unique(classifiers[,1]))
  # make factor for clutch
  clutch <- factor(classifiers[,3], levels = unique(classifiers[,3])) 
  # [1] "E14331" "E14518" "E15293" "E6328"  "E06238" "E10376" "E14319" "E14515" "E14372" "E5185" 
  # [11] "E6303"  "E6324"  "E10499" "E12643" "E13865" "E14555"
  
# reorder shape data to match classifiers file
  egg.gpa.surfs$coords <- egg.gpa.surfs$coords[,,rownames(classifiers)]
  
# Principal Components Analysis
  PCA <- plotTangentSpace(egg.gpa.surfs$coords, groups= species, warpgrids = F) 
  summary(PCA)

# Original alphabetical order of species gave PC1 the other way around, so swap these here for ease of reference
  PCA$pc.scores[,1] <- PCA$pc.scores[,1] * -1

# To plot a graph of proportion of variance explained by each PC
  pvar <- ((PCA$pc.summary)$sdev^2)/
    (sum((PCA$pc.summary)$sdev^2)) # calculate the proportion of variance
  names(pvar) <- c(1:length((PCA$pc.summary)$sdev))
  barplot(pvar, type = "b", xlab= "Principal Components", ylab = "% Variance")

# Set PCA axis labels
  xlab <- paste("Principal Component 1 ", "(", round(PCA$pc.summary$importance[2,1]*100, 1), "%)", sep="")
  ylab <- paste("Principal Component 2 ", "(", round(PCA$pc.summary$importance[2,2]*100, 1), "%)", sep="")

## PCA Figure
  # generates a set of different plotting colours for species
  col.gp <- c("#ED2024", "#3953A4", "#FAA41A", "#76CCE9") # hex codes for colours c("red", "blue", "yellow", "green") 
  names(col.gp) <- levels(species) # assign levels to these 4 colours
  col.gp <- col.gp[match(species, names(col.gp))] # clever code to match up each specimen with a colour according to its species

# generates a set of different plotting shapes for each clutch (4 clutches per species)
  pch.gp <- rep(c(24,22,25,21),4)
  names(pch.gp ) <- levels(clutch)
  pch.gp  <- pch.gp [match(clutch, names(pch.gp))]

# Plot PCA of PC1 vs PC2, shape is clutch, colour is species
  plot(PCA$pc.scores[,1], PCA$pc.scores[,2], cex=1.5, pch=pch.gp, bg = col.gp, 
       xlab=xlab, ylab=ylab, asp=T)

#######################################################

## Warp a mesh to the mean egg shape
# which is "most average" egg?
  findMeanSpec(egg.gpa.surfs$coords)
  # Colluricincia_harmonica_E6328_Egg2 
  # 10 
  ref <- mshape(egg.gpa.surfs$coords) # get mean shape

  meshcoords <- readmulti.nts("NTS/Colluricincia_harmonica_E6328_Egg2.nts") # import the RAW landmark coordinates digitized on the mesh
  mesh <- file2mesh("PLY/Colluricincia_harmonica_E6328_Egg2.ply") # need to rotate egg
  # need to rotate egg
  mesh <- pcAlign(mesh) # align by principal axes
  vb <- as.matrix(t(mesh$vb)[,-4])
  #Find top of egg - this is the half of the egg that has a smaller volume
  ans <- which.min(c(convhulln(vb[which(vb[,1] > 0),], options= "FA")$vol, 
                     convhulln(vb[which(vb[,1] < 0),], options= "FA")$vol)) # which is small volume?
  if(ans == 2) { vb[,1] <- vb[,1]*-1
                 mesh$vb <- rbind(t(vb), 1)
  } # if volume at pos values of x is larger, flip x axis
  remove(vb, ans) # cleans up environment
  # warp the mesh to the mean shape
  refmesh <- warpRefMesh(mesh, meshcoords[,,1], ref, color="#F0EAD6", centered=T) 
  # this will take a few moments, please wait patiently!
  # two rgl windows will have appeared. One is the imported mesh, and the other is this warped to the ref coords
  
  # plot the specimen with its landmarks and controls for excessive shininess
  plotspec(refmesh, ref, fixed=5, ptsize=0.01, 
           centered=F, lit = F, shininess = 50.0, specular=100) 

# Making figure for lmks
# Plot Eggs
  plot3d(refmesh, lit=T, box=F, axes=F, xlab="", ylab="", zlab="", specular = "#F0EAD6", ambient= "black", shininess = 100, alpha=0.6)
  points3d(ref[1:6, ], aspect = FALSE, size = 20, col = "red")
  points3d(ref[7:206, ], aspect = FALSE, size = 15, col = "black")
  # orient into position then...
  rgl.snapshot(filename="landmark figure wout axes.png", fmt = "png") 

# Warp the mean shaped egg (refmesh) to min and max of PCs 1 and 2
  # And Plot and save Eggs
  PC1max <- plotRefToTarget(ref, PCA$pc.shapes$PC1max, method="surface", 
                            mesh = refmesh, mag=1)  
  PC1min <- plotRefToTarget(ref, PCA$pc.shapes$PC1min, method="surface", 
                            mesh = refmesh, mag=1)
  PC2max <- plotRefToTarget(ref, PCA$pc.shapes$PC2max, method="surface", 
                            mesh = refmesh, mag=1)  
  PC2min <- plotRefToTarget(ref, PCA$pc.shapes$PC2min, method="surface", 
                            mesh = refmesh, mag=1)  

# Set rgl window for saving png files
  open3d();shade3d(PC1max) # open a mesh on rgl
  usrMat <- par3d()$viewport # maximise window to screen then use this comman to set window size
  zoom <- par3d()$zoom # zoom in first then use this command to set zoom

# now run these 8 lines of code at once:
  plot3d(PC1max, lit=T, box=F, axes=F, xlab="", ylab="", zlab="", specular = "#F0EAD6", ambient= "black", shininess = 100)
  rgl.snapshot(filename="PC1max.png", fmt = "png")
  plot3d(PC1min, lit=T, box=F, axes=F, xlab="", ylab="", zlab="", specular = "#F0EAD6", ambient= "black", shininess = 100)
  rgl.snapshot(filename="PC1min.png", fmt = "png")
  plot3d(PC2max, lit=T, box=F, axes=F, xlab="", ylab="", zlab="", specular = "#F0EAD6", ambient= "black", shininess = 100)
  rgl.snapshot(filename="PC2max.png", fmt = "png")
  plot3d(PC2min, lit=T, box=F, axes=F, xlab="", ylab="", zlab="", specular = "#F0EAD6", ambient= "black", shininess = 100)
  rgl.snapshot(filename="PC2min.png", fmt = "png")
  # and all the screenshots are saved to working directory!

#######################################################
 # Statistical Analyses
gdf <- geomorph.data.frame(coords = egg.gpa.surfs$coords, species = species, clutch = clutch)
# Does egg shape differ between species?
procD.lm(coords ~ species, iter =1000, data=gdf) # Procruses ANOVA
# Type I (Sequential) Sums of Squares and Cross-products
# Randomized Residual Permutation Procedure Used
# 1001 Permutations
# 
#             Df       SS         MS     Rsq      F      Z   Pr(>F)    
#   species    3 0.008318 0.00277278 0.19978 4.2441 2.6329 0.000999 ***
#   Residuals 51 0.033319 0.00065332 0.80022                           
#   Total     54 0.041638                                               
# ---
#   Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1

# However given that we know the nested, hierarchical structure of the data, we should instead do the following:

nested.update(procD.lm(coords ~ species / clutch, iter =1000, data=gdf), ~species / clutch)
# Type I (Sequential) Sums of Squares and Cross-products
# Randomized Residual Permutation Procedure Used
# 1001 Permutations
# 
# *** F values, Z scores, and P values updated for nested effects
# 
#                   Df       SS         MS     Rsq      F      Z   Pr(>F)    
#   species         3 0.008318 0.00277278 0.19978 1.4768 0.4613 0.324675    
#   species:clutch 12 0.022531 0.00187758 0.54112 6.7875 6.1576 0.000999 ***
#   Residuals      39 0.010788 0.00027662 0.25910                           
#   Total          54 0.041638                                      
# ---
#   Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1


# Disparity of each clutch
clutch.disp <- morphol.disparity(coords ~ clutch, 
                                 iter = 1, data=gdf)$Procrustes.var # save just the Procrustes Variance for each clutch
spp.col <- rep(c("#ED2024", "#3953A4", "#FAA41A", "#76CCE9"), each=4) # make a new colour vector based on the colours of col.gp

# Disparity of each species
spp.disp <- morphol.disparity(coords ~ species, iter = 1000, data=gdf)
# Randomized Residual Permutation Procedure Used
# 1001 Permutations
# 
# Procrustes variances for defined groups
# Colluricincia      Neochmia  Acanthagenys       Malurus 
# 0.0007818705  0.0006433520  0.0008536290  0.0002518307 
# 
# 
# Pairwise absolute differences between variances
#               Colluricincia     Neochmia Acanthagenys      Malurus
# Colluricincia  0.000000e+00 0.0001385184 7.175854e-05 0.0005300397
# Neochmia       1.385184e-04 0.0000000000 2.102770e-04 0.0003915213
# Acanthagenys   7.175854e-05 0.0002102770 0.000000e+00 0.0006017983
# Malurus        5.300397e-04 0.0003915213 6.017983e-04 0.0000000000
# 
# 
# P-Values
#               Colluricincia   Neochmia Acanthagenys    Malurus
# Colluricincia    1.00000000 0.50349650   0.78821179 0.02197802
# Neochmia         0.50349650 1.00000000   0.36463536 0.05194805
# Acanthagenys     0.78821179 0.36463536   1.00000000 0.01198801
# Malurus          0.02197802 0.05194805   0.01198801 1.00000000

## Figure Disparity by clutch and species
par(mfrow = c(1, 2), mar=c(5,4,4,2))
ylim <- c(0, max(c(clutch.disp, spp.disp$Procrustes.var)))
# Plot the disparity of each species
barplot(spp.disp$Procrustes.var, ylab = "Procrustes Variance", xlab= "Species",cex.names=0.5, col = c("#ED2024", "#3953A4", "#FAA41A", "#76CCE9"), ylim= ylim, las=0)
par(mar=c(5,0,4,2))
# Plot the disparity of each clutch, colour by species
barplot(clutch.disp, ylab = NULL, xlab= "Clutch", col = spp.col, cex.names=0.5, ylim= ylim, las=3)


# Does mean clutch disparity differ between species?
spp <- rep(levels(species), each=4)
summary.aov(lm(clutch.disp ~ spp))
#             Df    Sum Sq   Mean Sq F value Pr(>F)
# spp          3 5.620e-09 1.873e-09   0.184  0.906
# Residuals   12 1.225e-07 1.021e-08      

summary(procD.lm(clutch.disp ~ spp))
# Type I (Sequential) Sums of Squares and Cross-products
# Randomized Residual Permutation Procedure Used
# 1000 Permutations
# 
#           Df         SS         MS      Rsq      F       Z Pr(>F)
# spp        3 5.6200e-09 1.8735e-09 0.04388 0.1836 -1.3118  0.909
# Residuals 12 1.2247e-07 1.0205e-08 0.95612                      
# Total     15 1.2809e-07 

# Disparity of clutches for each species
# "Acanthagenys"
acan.gdf <- geomorph.data.frame(coords = egg.gpa.surfs$coords[,,which(species == "Acanthagenys")],
                                spec = as.factor(classifiers[which(species == "Acanthagenys"),3]))
acan.disp <- morphol.disparity(coords ~ spec, iter = 1000, data=acan.gdf)
# concatenate into a data table:
X <- matrix(NA, ncol = 4, nrow=4, dimnames = dimnames(acan.disp$PV.dist.Pval))
X[upper.tri(X)] <- acan.disp$PV.dist.Pval[upper.tri(acan.disp$PV.dist.Pval)] 
X[lower.tri(X)] <- acan.disp$PV.dist[lower.tri(acan.disp$PV.dist)] 
write.csv(X, "Acanthagenys clutch disparity results.csv", na="-")

# "Colluricincia"
coll.gdf <- geomorph.data.frame(coords = egg.gpa.surfs$coords[,,which(species == "Colluricincia")],
                                spec = as.factor(classifiers[which(species == "Colluricincia"),3]))
coll.disp <- morphol.disparity(coords ~ spec, iter = 1000, data=coll.gdf)
# concatenate into a data table:
Y <- matrix(NA, ncol = 4, nrow=4, dimnames = dimnames(coll.disp$PV.dist.Pval))
Y[upper.tri(Y)] <- coll.disp$PV.dist.Pval[upper.tri(coll.disp$PV.dist.Pval)] 
Y[lower.tri(Y)] <- coll.disp$PV.dist[lower.tri(coll.disp$PV.dist)] 
write.csv(Y, "Colluricincia clutch disparity results.csv", na="-")

# "Malurus"
Mal.gdf <- geomorph.data.frame(coords = egg.gpa.surfs$coords[,,which(species == "Malurus")],
                               spec = as.factor(classifiers[which(species == "Malurus"),3]))
Mal.disp <- morphol.disparity(coords ~ spec, iter = 1000, data=Mal.gdf)
# concatenate into a data table:
Z <- matrix(NA, ncol = 4, nrow=4, dimnames = dimnames(Mal.disp$PV.dist.Pval))
Z[upper.tri(Z)] <- Mal.disp$PV.dist.Pval[upper.tri(Mal.disp$PV.dist.Pval)] 
Z[lower.tri(Z)] <- Mal.disp$PV.dist[lower.tri(Mal.disp$PV.dist)] 
write.csv(Z, "Malurus clutch disparity results.csv", na="-")

# "Neochmia"
neo.gdf <- geomorph.data.frame(coords = egg.gpa.surfs$coords[,,which(species == "Neochmia")],
                               spec = as.factor(classifiers[which(species == "Neochmia"),3]))
neo.disp <- morphol.disparity(coords ~ spec, iter = 1000, data=neo.gdf)
# concatenate into a data table:
W <- matrix(NA, ncol = 4, nrow=4, dimnames = dimnames(neo.disp$PV.dist.Pval))
W[upper.tri(W)] <- neo.disp$PV.dist.Pval[upper.tri(neo.disp$PV.dist.Pval)] 
W[lower.tri(W)] <- neo.disp$PV.dist[lower.tri(neo.disp$PV.dist)] 
write.csv(W, "Neochmia clutch disparity results.csv", na="-")

################################################################################################
# Calculating elongation and asymmetry from the shapes
# Elogantion is calculated as the ratio of egg length to breadth (maximum diameter) describes the divergence of an ellipse (flattening) from a circle

# length is the distance between lanmdarks 1-2
# width is the distance between landmarks 3-4
# hypotenuses are the distance between lower vertex of egg (lmk 1) and the first 2 equitoral lmks
lmks <- matrix(c(1:4,1,3,1,4), ncol=2, byrow=TRUE, 
               dimnames = list(c("eggl", "eggw", "hyp","hyp2"),c("start", "end")))
lengths <- interlmkdist(egg.gpa.surfs$coords, lmks)
lengths <- data.frame(lengths, "elongation" = lengths[,1] /lengths[,2])
lengths <- data.frame(lengths, "av.hyp" = (lengths$hyp + lengths$hyp2)/2)

# The index for egg asymmetry is calculated based on the ratio between the distance from the lower vertex of the egg to the point where the polar axis intersects the equatorial axis to egg length
# so distance from lmk 1 to the midway between lmks 3&4

# using trigonometry
# the adjacent length is half egg width
lengths <- data.frame(lengths, "adj" = lengths$eggw/2)
# the angle is arc-cos of adj/hyp (need to convert from radians to degrees)
lengths <- data.frame(lengths, "angle" = (180*acos(lengths$adj/lengths$av.hyp))/pi)
# distance from the lower vertex of the egg to the point where the polar axis intersects the equatorial axis is therefore the "opposite", which is hypoteneuse * sin(angle)
lengths <- data.frame(lengths, "asymmetry" = (lengths$hyp * sin(lengths$angle))/lengths$eggl)

plot(lengths$elongation, lengths$asymmetry, cex=1.5, pch=pch.gp, bg = col.gp)

# How similar are the two morphospaces?
vegan::mantel(dist(cbind(lengths$elongation, lengths$asymmetry)),
              dist(PCA$pc.scores))
# Mantel statistic r: 0.4095 
# Significance: 0.001 
# Not very.

# How similar are specimen distrbutions along the two axes?
cor(PCA$pc.scores[,1], lengths$elongation)
# 0.9922384 # Strong correlation between elongation and PC1

cor(PCA$pc.scores[,2], lengths$asymmetry)
# 0.6592932 # moderate

