# These functions are based on the 3D digitizing function from the geomorph library (https://cran.r-project.org/web/packages/geomorph/)
# Source file written by Emma Sherratt (emma.sherratt@gmail.com)

# Load libraries required
library(rgl) ; library(Morpho) ; library(geometry)

# Function 1 - Build Template of surface-sliding semilandmarks
  # Arguments
  # spec: Name of surface file (e.g. ply file), as an object of class shape3d/mesh3d
  # surface.sliders: A numerical value for the number of template surface sliding semilandmarks desired
  # ptsize: A numerical value for the size to plot the mesh points (vertices), e.g. 0.1 for dense meshes, 3 for sparse meshes
  # Function is based on buildtemplate{geomorph}.

build.egg.template<-function(spec, surface.sliders, ptsize = 1)    {
  spec.name<-deparse(substitute(spec))
  spec <- pcAlign(spec)
  specimen <- as.matrix(t(spec$vb)[,-4])
  #Find top of egg - this is the half of the egg that has a smaller volume
  ans <- which.min(c(convhulln(specimen[which(specimen[,1] > 0),], options= "FA")$vol, 
                     convhulln(specimen[which(specimen[,1] < 0),], options= "FA")$vol)) # which is small volume?
  if(ans == 2) { specimen[,1] <- specimen[,1]*-1
                 spec$vb <- rbind(t(specimen), 1)
                 } # if volume at pos values of x is larger, flip x axis
  mesh <- spec 
  mesh$material <- "grey" 
  fixed = 6
  #digitize fixed points
    # Save lmk coords
    selected <- matrix(NA, ncol=3, nrow=fixed)
    selected[1,] <- specimen[which.min(specimen[,1]),] ; selected[2,] <- specimen[which.max(specimen[,1]),]
    selected[3,] <- specimen[which.min(specimen[,2]),] ; selected[4,] <- specimen[which.max(specimen[,2]),]
    selected[5,] <- specimen[which.min(specimen[,3]),] ; selected[6,] <- specimen[which.max(specimen[,3]),]
    # Save lmk addresses
    fix <- matrix(NA, nrow=fixed)
    fix[1] <- which.min(specimen[,1]) ; fix[2] <- which.max(specimen[,1])
    fix[3] <- which.min(specimen[,2]) ; fix[4] <- which.max(specimen[,2])
    fix[5] <- which.min(specimen[,3]) ; fix[6] <- which.max(specimen[,3])
  # plot the mesh with the fixed points at the poles
  plot3d(specimen[,1],specimen[,2],specimen[,3],size=ptsize,aspect=FALSE)
  shade3d(mesh, add=TRUE)
  points3d(selected[,1],selected[,2],selected[,3],size=10,color="red",add=TRUE)
  # Calculate semilandmarks
  surfs<-specimen[-fix,]
  template<-rbind(selected,kmeans(x=surfs,centers=surface.sliders,iter.max=100)$centers)
  points3d(template[-(1:fixed),1],template[-(1:fixed),2],template[-(1:fixed),3],size=10,col="blue")
  write.table(template,file="template.txt",row.names=F,col.names=TRUE)
}

# Function 2 - Digitize all eggs using template of surface-sliding semilandmarks
  # Arguments
  # spec: Name of surface file (e.g. ply file), as an object of class shape3d/mesh3d
  # ptsize: A numerical value for the size to plot the mesh points (vertices), e.g. 0.1 for dense meshes, 3 for sparse meshes
  # spec.name: The name of the egg specimen (which is usually the file name without the suffix)
  # based on digitsurface{geomorph}

digit.egg.surface<-function(spec, ptsize = 1, spec.name)    {
  spec.name<-spec.name
  spec <- pcAlign(spec)
  specimen <- as.matrix(t(spec$vb)[,-4])
  #Find top of egg - this is the half of the egg that has a smaller volume
  ans <- which.min(c(convhulln(specimen[which(specimen[,1] > 0),], options= "FA")$vol, 
                     convhulln(specimen[which(specimen[,1] < 0),], options= "FA")$vol)) # which is small volume?
  if(ans == 2) { specimen[,1] <- specimen[,1]*-1
                 spec$vb <- rbind(t(specimen), 1)
  } # if volume at pos values of x is larger, flip x axis
  mesh <- spec 
  mesh$material <- "gray" 
  fixed = 6
  
  #add fixed points
    # save lmk coords
    selected <- matrix(NA, ncol=3, nrow=6)
    selected[1,] <- specimen[which.min(specimen[,1]),] ; selected[2,] <- specimen[which.max(specimen[,1]),]
    selected[3,] <- specimen[which.min(specimen[,2]),] ; selected[4,] <- specimen[which.max(specimen[,2]),]
    selected[5,] <- specimen[which.min(specimen[,3]),] ; selected[6,] <- specimen[which.max(specimen[,3]),]
    # save lmk address
    fix <- matrix(NA, nrow=6)
    fix[1] <- which.min(specimen[,1]) ; fix[2] <- which.max(specimen[,1])
    fix[3] <- which.min(specimen[,2]) ; fix[4] <- which.max(specimen[,2])
    fix[5] <- which.min(specimen[,3]) ; fix[6] <- which.max(specimen[,3])
  # plot the mesh with the fixed points at the poles
  plot3d(specimen[,1],specimen[,2],specimen[,3],size=ptsize,aspect=FALSE)
  shade3d(mesh, add=TRUE)
  points3d(selected[,1],selected[,2],selected[,3],size=10,color="red",add=TRUE)
  # Read in template and fit semilandmarks
  template<-as.matrix(read.table("template.txt",header=TRUE))
  specimen<-trans(as.matrix(specimen))
  template<-trans(template)*(csize(specimen[fix,])[[1]]/csize(template[(1:fixed),])[[1]])  
  template<-template%*%(pPsup(template[(1:fixed),],specimen[fix,]))[[3]] 
  template.tps<-tps2d3d(template[-(1:fixed),],template[(1:fixed),],specimen[fix,])             
  spec.surfs<-specimen[-fix,]
  nei<-numeric(dim(template.tps)[1])
  sliders<-matrix(NA,nrow=dim(template.tps)[1],ncol=3)
  for (i in 1:dim(template.tps)[1])     {
    nei[i]<-which.min(sqrt((template.tps[i,1]-spec.surfs[,1])^2+(template.tps[i,2]-spec.surfs[,2])^2+(template.tps[i,3]-spec.surfs[,3])^2))[1] #3D NN
    sliders[i,]<-spec.surfs[nei[i],]
    spec.surfs<-spec.surfs[-nei[i],]  
  }
  clear3d();plot3d(specimen[,1],specimen[,2],specimen[,3],size=ptsize,aspect=FALSE)
  if (!is.null(mesh)) { 
    mesh$vb <- rbind(t(specimen), 1)
    shade3d(mesh, add=TRUE) 
  }
  points3d(specimen[fix,],col="red",size=10)
  points3d(template.tps,col="blue",size=10)
  points3d(sliders[,1:3],size=10,col="green")
  selected.out <- rbind(specimen[fix,],sliders)
  writeland.nts(selected.out, spec.name, comment=NULL)
}

# Write .nts file
# A: A n-x-p matrix of the output coordinates.

writeland.nts <- function(A, spec.name, comment=NULL){
  ntsfile=paste(spec.name,".nts",sep="")
  file.create(file=ntsfile)
  if(is.null(comment)){
    cat(paste('"',spec.name,sep=""),file= ntsfile,sep="\n",append=TRUE)
  }
  else if(!is.null(comment)){
    cat(paste('"',spec.name,sep=""),file= ntsfile,sep="\n")
    cat(paste('"',comment,sep=""),file= ntsfile,sep="\n",append=TRUE)
  }
  dims <- dim(A)
  if (dims[2] == 2){
    cat(paste(1,dims[1],2,0,"dim=2"),file= ntsfile,sep="\n",append=TRUE)
  }
  else if (dims[2] == 3){
    cat(paste(1,dims[1],3,0, "dim=3"),file= ntsfile,sep="\n",append=TRUE)
  }
  write.table(A ,file= ntsfile,col.names = FALSE, row.names = FALSE,sep="  ",append=TRUE)
}

trans<-function(A){scale(A,scale=FALSE)}     #J. Claude 2008

csize<-function(A)  			#J. Claude 2008
{p<-dim(A)[1]
 size<-sqrt(sum(apply(A,2,var))*(p-1))
 list("centroid_size"=size,"scaled"=A/size)}

pPsup<-function(M1,M2){  			#J. Claude 2008# MODIFIED BY Dean C Adams (DCA): 2/2014
  k<-ncol(M1)
  Z1<-trans(csize(M1)[[2]])
  Z2<-trans(csize(M2)[[2]])
  sv<-svd(t(Z2)%*%Z1)
  U<-sv$v; V<-sv$u; Delt<-sv$d
  sig<-sign(det(t(Z2)%*%Z1))
  Delt[k]<-sig*abs(Delt[k]); V[,k]<-sig*V[,k]
  Gam<-U%*%t(V)
  # beta<-sum(Delt)   #commented out: retain size-scaling (DCA)
  #  list(Mp1=beta*Z1%*%Gam,Mp2=Z2,rotation=Gam,scale=beta,
  #       df=sqrt(1-beta^2))}
  list(Mp1=Z1%*%Gam,Mp2=Z2,rotation=Gam)}    #simplify output and remove size re-scaling (DCA) 

TransRot<-function(M1,M2){  			#DCA: 4, 2014 translate, rotate one specimen to another
  k<-ncol(M1)
  Z1<-trans(M1)
  Z2<-trans(M2)
  sv<-svd(t(Z2)%*%Z1)
  U<-sv$v; V<-sv$u; Delt<-sv$d
  sig<-sign(det(t(Z2)%*%Z1))
  Delt[k]<-sig*abs(Delt[k]); V[,k]<-sig*V[,k]
  Gam<-U%*%t(V)
  list(Mp1=Z1%*%Gam,Mp2=Z2,rotation=Gam)}    

tps2d3d<-function(M, matr, matt){  	#DCA: altered from J. Claude 2008  
  p<-dim(matr)[1]; k<-dim(matr)[2];q<-dim(M)[1]
  Pdist<-as.matrix(dist(matr))
  ifelse(k==2,P<-Pdist^2*log(Pdist^2),P<-Pdist) 
  P[which(is.na(P))]<-0
  Q<-cbind(1, matr)
  L<-rbind(cbind(P,Q), cbind(t(Q),matrix(0,k+1,k+1)))
  m2<-rbind(matt, matrix(0, k+1, k))   
  coefx<-solve(L)%*%m2[,1]
  coefy<-solve(L)%*%m2[,2]
  if(k==3){coefz<-solve(L)%*%m2[,3]}
  fx<-function(matr, M, coef){
    Xn<-numeric(q)
    for (i in 1:q){
      Z<-apply((matr-matrix(M[i,],p,k,byrow=TRUE))^2,1,sum)  
      ifelse(k==2,Z1<-Z*log(Z),Z1<-sqrt(Z)); Z1[which(is.na(Z1))]<-0
      ifelse(k==2,Xn[i]<-coef[p+1]+coef[p+2]*M[i,1]+coef[p+3]*M[i,2]+sum(coef[1:p]*Z1),
             Xn[i]<-coef[p+1]+coef[p+2]*M[i,1]+coef[p+3]*M[i,2]+coef[p+4]*M[i,3]+sum(coef[1:p]*Z1))
    }    
    Xn}
  matg<-matrix(NA, q, k)
  matg[,1]<-fx(matr, M, coefx)
  matg[,2]<-fx(matr, M, coefy)
  if(k==3){matg[,3]<-fx(matr, M, coefz)}  
  matg
}

